@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -38,8 +37,6 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import logging
 log = logging.getLogger(__name__)
 
-from bokeh.util.api import general, dev ; general, dev
-
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
@@ -56,11 +53,18 @@ from .handler import Handler
 # Globals and constants
 #-----------------------------------------------------------------------------
 
+__all__ = (
+    'FunctionHandler',
+)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-@general((1,0,0))
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
 class FunctionHandler(Handler):
     ''' A Handler that accepts a plain python function to use for modifying
     Bokeh Documents.
@@ -109,7 +113,6 @@ class FunctionHandler(Handler):
     # Properties --------------------------------------------------------------
 
     @property
-    @general((1,0,0))
     def safe_to_fork(self):
         ''' Whether it is still safe for the Bokeh server to fork new workers.
 
@@ -120,7 +123,6 @@ class FunctionHandler(Handler):
 
     # Public methods ----------------------------------------------------------
 
-    @general((1,0,0))
     def modify_document(self, doc):
         ''' Execute the configured ``func`` to modify the document.
 
@@ -131,10 +133,6 @@ class FunctionHandler(Handler):
         self._func(doc)
         self._safe_to_fork = False
 
-#-----------------------------------------------------------------------------
-# Dev API
-#-----------------------------------------------------------------------------
-
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
