@@ -41,7 +41,7 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 class HSL(Color):
-    ''' Represent colors by specifying their Hue, Saturation, and Value.
+    ''' Represent colors by specifying their Hue, Saturation, and lightness.
 
     Alpha values may also optionally be provided. Otherwise, alpha values
     default to 1.
@@ -53,13 +53,13 @@ class HSL(Color):
 
         Args:
             h (int) :
-                The Hue, in [0, 255]
+                The Hue, in [0, 360]
 
-            h (int) :
+            s (int) :
                 The Saturation, in [0, 1]
 
-            h (int) :
-                The Value, in [0, 1]
+            l (int) :
+                The lightness, in [0, 1]
 
             a (float, optional) :
                 An alpha value for this color in [0, 1] (default: 1.0)
