@@ -83,27 +83,24 @@ Otherwise, You can see an index of all running applications at the server root:
 This index can be disabled with the ``--disable-index`` option, and the redirect
 behavior can be disabled with the ``--disable-index-redirect`` option.
 
-Network Configuration
-~~~~~~~~~~~~~~~~~~~~~
-
-To control the port that the Bokeh server listens on, use the ``--port``
-argument:
+Another way to run multiple applications is using glob notation to indicate
+that all the files matching a particular pattern should be served.
 
 .. code-block:: sh
 
-    bokeh serve app_script.py --port 8080
+    bokeh serve *.py
 
-To listen on an arbitrary port, pass ``0`` as the port number.  The actual
-port number will be logged at startup.
+Command line shells will normally expand the ``*.py`` automatically. However,
+if you are starting a Bokeh server programmatically, then filename arguments
+with globs may not be expanded by the shell. In situations like this, the
+``--glob`` flag may be used to make the Bokeh server explicitly perform globbing:
 
-Similarly, a specific network address can be specified with the
-``--address`` argument. For example:
+.. code-block:: python
 
-.. code-block:: sh
+    subprocess.call(["bokeh", "serve", "--glob", "*.py"])
 
-    bokeh serve app_script.py --address 0.0.0.0
-
-will have the Bokeh server listen all available network addresses.
+Application Configuration
+~~~~~~~~~~~~~~~~~~~~~~~~~
 
 Bokeh server can fork the underlying tornado server into multiprocess.  This is
 useful when trying to handle multiple connections especially in the context of
@@ -119,19 +116,6 @@ Note that due to limitations inherent in Tornado, Windows does not support
 ``--num-procs`` values greater than one! In this case consider running multiple
 Bokeh server instances behind a load balancer.
 
-By default, cross site connections to the Bokeh server websocket are not
-allowed. You can enable websocket connections originating from additional
-hosts by specifying them with the ``BOKEH_ALLOW_WS_ORIGIN`` environment variable
-or the ``--allow-websocket-origin`` option:
-
-.. code-block:: sh
-
-    bokeh serve app_script.py --allow-websocket-origin foo.com:8081
-
-It is possible to specify multiple allowed websocket origins by adding
-the ``--allow-websocket-origin`` option multiple times and to provide a
-comma separated list of hosts to ``BOKEH_ALLOW_WS_ORIGIN``
-
 The Bokeh server can also add an optional prefix to all URL paths.
 This can often be useful in conjunction with "reverse proxy" setups.
 
@@ -155,6 +139,241 @@ To configure this feature, set the ``--keep-alive`` option:
 The value is specified in milliseconds. The default keep-alive interval
 is 37 seconds. Give a value of 0 to disable keep-alive pings.
 
+Network Configuration
+~~~~~~~~~~~~~~~~~~~~~
+
+To control the port that the Bokeh server listens on, use the ``--port``
+argument:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py --port 8080
+
+To listen on an arbitrary port, pass ``0`` as the port number.  The actual
+port number will be logged at startup.
+
+Similarly, a specific network address can be specified with the
+``--address`` argument. For example:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py --address 0.0.0.0
+
+will have the Bokeh server listen all available network addresses.
+
+By default, cross site connections to the Bokeh server websocket are not
+allowed. You can enable websocket connections originating from additional
+hosts by specifying them with the ``BOKEH_ALLOW_WS_ORIGIN`` environment variable
+or the ``--allow-websocket-origin`` option:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py --allow-websocket-origin foo.com:8081
+
+It is possible to specify multiple allowed websocket origins by adding
+the ``--allow-websocket-origin`` option multiple times and to provide a
+comma separated list of hosts to ``BOKEH_ALLOW_WS_ORIGIN``
+
+To have the Bokeh server override the remote IP and URI scheme/protocol for
+all requests with ``X-Real-Ip``, ``X-Forwarded-For``, ``X-Scheme``,
+``X-Forwarded-Proto``  headers (if they are provided), set the
+``--use-xheaders`` option:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py --use-xheaders
+
+This is typically needed when running a Bokeh server behind a reverse proxy
+that is SSL-terminated.
+
+.. warning::
+    It is not advised to set this option on a Bokeh server directly facing
+    the Internet.
+
+A Bokeh server can also terminate SSL connections directly by specifying the
+path to a single file in PEM format containing the certificate as well as any
+number of CA certificates needed to establish the certificate's authenticity:
+
+.. code-block:: sh
+
+    bokeh serve --ssl-certfile /path/to/cert.pem
+
+Alternatively, the path may also be supplied by setting the environment
+variable ``BOKEH_SSL_CERTFILE``.
+
+If the private key is stored separately, its location may be supplied by
+setting the ``--ssl-keyfile`` command line argument, or by setting the
+``BOKEH_SSL_KEYFILE`` evironment variable. If a password is required for the
+private key, it should be supplied by setting the ``BOKEH_SSL_PASSWORD``
+environment variable.
+
+Session ID Options
+~~~~~~~~~~~~~~~~~~
+
+Typically, each browser tab connected to a Bokeh server will have its own
+session ID. When the server generates an ID, it will make it cryptographically
+unguessable. This keeps users from accessing one another's sessions.
+
+To control who can use a Bokeh application, the server can sign session IDs
+with a secret key and reject "made up" session names. There are three modes,
+controlled by the ``--session-ids`` argument:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py --session-ids signed
+
+The available modes are: {SESSION_ID_MODES}
+
+In ``unsigned`` mode, the server will accept any session ID provided to it in
+the URL. For example, ``http://localhost/app_script?bokeh-session-id=foo`` will
+create a session ``foo``. In ``unsigned`` mode, if the session ID isn't
+provided with ``?bokeh-session-id=`` in the URL, the server will still generate
+a cryptographically-unguessable ID. However, the server allows clients to
+create guessable or deliberately-shared sessions if they want to.
+
+``unsigned`` mode is most useful when the server is running locally for
+development, for example you can have multiple processes access a fixed session
+name such as ``default``. ``unsigned`` mode is also convenient because there's
+no need to generate or configure a secret key.
+
+In ``signed`` mode, the session ID must be in a special format and signed with
+a secret key. Attempts to use the application with an invalid session ID will
+fail, but if no ``?bokeh-session-id=`` parameter is provided, the server will
+generate a fresh, signed session ID. The result of ``signed`` mode is that only
+secure session IDs are allowed but anyone can connect to the server.
+
+In ``external-signed`` mode, the session ID must be signed but the server
+itself won't generate a session ID; the ``?bokeh-session-id=`` parameter will
+be required. To use this mode, an external process (such as another web app)
+would use the function ``bokeh.util.session_id.generate_session_id()`` to
+create valid session IDs. The external process and the Bokeh server must share
+the same ``BOKEH_SECRET_KEY`` environment variable.
+
+``external-signed`` mode is useful if you want another process to authenticate
+access to the Bokeh server. If someone is permitted to use a Bokeh application,
+you would generate a session ID for them, then redirect them to the Bokeh
+server with that valid session ID. If you don't generate a session ID for
+someone, then they can't load the app from the Bokeh server.
+
+In both ``signed`` and ``external-signed`` mode, the secret key must be kept
+secret; anyone with the key can generate a valid session ID.
+
+The secret key should be set in a ``BOKEH_SECRET_KEY`` environment variable and
+should be a cryptographically random string with at least 256 bits (32 bytes)
+of entropy. The ``bokeh secret`` command can generate new secret keys.
+
+Authentication Options
+~~~~~~~~~~~~~~~~~~~~~~
+
+The Bokeh server can be configured to only allow connections in case there is
+a properly authenticated user. This is accomplished by providing the path to
+a module that implements the necessary functions on the command line:
+
+.. code-block:: sh
+
+    bokeh serve --auth-module=/path/to/auth.py
+
+or by setting the ``BOKEH_AUTH_MODULE`` environment variable.
+
+The module must contain *one* of the following two functions that will return
+the current user (or None):
+
+.. code-block:: python
+
+    def get_user(request_handler):
+        pass
+
+    async def get_user_async(request_handler):
+        pass
+
+The function is passed the Tornado ``RequestHandler`` and can inspect cookies
+or request headers to determine the authenticated user. If there is no valid
+authenticated user, these functions should return None.
+
+Additionally, the module must specify where to redirect unauthenticated users.
+It must contain either:
+
+* a module attribute ``login_url`` and (optionally) a ``LoginHandler`` class
+* a function definition for ``get_login_url``
+
+.. code-block:: python
+
+    login_url = "..."
+
+    class LoginHandler(RequestHandler):
+        pass
+
+    def get_login_url(request_handler):
+        pass
+
+When a relative ``login_url`` is given, an optional ``LoginHandler`` class may
+also be provided, and it will be installed as a route on the Bokeh server
+automatically.
+
+The ``get_login_url`` function is useful in cases where the login URL must
+vary based on the request, or cookies, etc. It is not possible to specify a
+``LoginHandler`` when ``get_url_function`` is defined.
+
+Analogous to the login options, optional ``logout_url`` and ``LogoutHandler``
+values may be define an endpoint for logging users out.
+
+If no auth module is provided, then a default user will be assumed, and no
+authentication will be required to access Bokeh server endpoints.
+
+.. warning::
+    The contents of the auth module will be executed!
+
+Bokeh can also enable the use of Tornado's XFRF cookie protection. To turn this
+feature on, use the ``--enable-xsrf-cookies`` option, or set the environment
+variable ``BOKEH_XSRF_COOKIES=yes``. If this setting is enabled, any PUT, POST,
+or DELETE operations on custom or login handlers must be instrumented properly
+in order to function. Typically, this means adding the ``xsrf_form_html()``
+module to HTML form submissions templates. For full details, see:
+
+    https://www.tornadoweb.org/en/stable/guide/security.html#cross-site-request-forgery-protection
+
+Session Expiration Options
+~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+To configure how often to check for unused sessions. set the
+``--check-unused-sessions`` option:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py --check-unused-sessions 10000
+
+The value is specified in milliseconds. The default interval for checking for
+unused sessions is 17 seconds. Only positive integer values are accepted.
+
+To configure how often unused sessions last. set the
+``--unused-session-lifetime`` option:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py --unused-session-lifetime 60000
+
+The value is specified in milliseconds. The default lifetime interval for
+unused sessions is 15 seconds. Only positive integer values are accepted.
+
+Diagnostic Options
+~~~~~~~~~~~~~~~~~~
+
+The logging level can be controlled by the ``--log-level`` argument:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py --log-level debug
+
+The available log levels are: {LOGLEVELS}
+
+The log format can be controlled by the ``--log-format`` argument:
+
+.. code-block:: sh
+
+    bokeh serve app_script.py --log-format "%(levelname)s: %(message)s"
+
+The default log format is ``"{DEFAULT_LOG_FORMAT}"``
+
 To control how often statistic logs are written, set the
 ``--stats-log-frequency`` option:
 
@@ -176,132 +395,6 @@ set the ``--mem-log-frequency`` option:
 The value is specified in milliseconds. The default interval for
 logging stats is 0 (disabled). Only positive integer values are accepted.
 
-To have the Bokeh server override the remote IP and URI scheme/protocol for
-all requests with ``X-Real-Ip``, ``X-Forwarded-For``, ``X-Scheme``,
-``X-Forwarded-Proto``  headers (if they are provided), set the
-``--use-xheaders`` option:
-
-.. code-block:: sh
-
-    bokeh serve app_script.py --use-xheaders
-
-This is typically needed when running a Bokeh server behind a reverse proxy
-that is SSL-terminated.
-
-.. warning::
-    It is not advised to set this option on a Bokeh server directly facing
-    the Internet.
-
-Session ID Options
-~~~~~~~~~~~~~~~~~~
-
-Typically, each browser tab connected to a Bokeh server will have
-its own session ID. When the server generates an ID, it will make
-it cryptographically unguessable. This keeps users from accessing
-one another's sessions.
-
-To control who can use a Bokeh application, the server can sign
-sessions with a secret key and reject "made up" session
-names. There are three modes, controlled by the ``--session-ids``
-argument:
-
-.. code-block:: sh
-
-    bokeh serve app_script.py --session-ids signed
-
-The available modes are: {SESSION_ID_MODES}
-
-In ``unsigned`` mode, the server will accept any session ID
-provided to it in the URL. For example,
-``http://localhost/app_script?bokeh-session-id=foo`` will create a
-session ``foo``. In ``unsigned`` mode, if the session ID isn't
-provided with ``?bokeh-session-id=`` in the URL, the server will
-still generate a cryptographically-unguessable ID. However, the
-server allows clients to create guessable or deliberately-shared
-sessions if they want to.
-
-``unsigned`` mode is most useful when the server is running
-locally for development, for example you can have multiple
-processes access a fixed session name such as
-``default``. ``unsigned`` mode is also convenient because there's
-no need to generate or configure a secret key.
-
-In ``signed`` mode, the session ID must be in a special format and
-signed with a secret key. Attempts to use the application with an
-invalid session ID will fail, but if no ``?bokeh-session-id=``
-parameter is provided, the server will generate a fresh, signed
-session ID. The result of ``signed`` mode is that only secure
-session IDs are allowed but anyone can connect to the server.
-
-In ``external-signed`` mode, the session ID must be signed but the
-server itself won't generate a session ID; the
-``?bokeh-session-id=`` parameter will be required. To use this
-mode, you would need some sort of external process (such as
-another web app) which would use the
-``bokeh.util.session_id.generate_session_id()`` function to create
-valid session IDs. The external process and the Bokeh server must
-share the same ``BOKEH_SECRET_KEY`` environment variable.
-
-``external-signed`` mode is useful if you want another process to
-authenticate access to the Bokeh server; if someone is permitted
-to use the Bokeh application, you would generate a session ID for
-them, then redirect them to the Bokeh server with that valid
-session ID. If you don't generate a session ID for someone, then
-they can't load the app from the Bokeh server.
-
-In both ``signed`` and ``external-signed`` mode, the secret key
-must be kept secret; anyone with the key can generate a valid
-session ID.
-
-The secret key should be set in a ``BOKEH_SECRET_KEY`` environment
-variable and should be a cryptographically random string with at
-least 256 bits (32 bytes) of entropy.  You can generate a new
-secret key with the ``bokeh secret`` command.
-
-Session Expiration Options
-~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-To configure how often to check for unused sessions. set the
---check-unused-sessions option:
-
-.. code-block:: sh
-
-    bokeh serve app_script.py --check-unused-sessions 10000
-
-The value is specified in milliseconds. The default interval for
-checking for unused sessions is 17 seconds. Only positive integer
-values are accepted.
-
-To configure how often unused sessions last. set the
---unused-session-lifetime option:
-
-.. code-block:: sh
-
-    bokeh serve app_script.py --unused-session-lifetime 60000
-
-The value is specified in milliseconds. The default lifetime interval
-for unused sessions is 15 seconds. Only positive integer values are
-accepted.
-
-Logging Options
-~~~~~~~~~~~~~~~
-
-The logging level can be controlled by the ``--log-level`` argument:
-
-.. code-block:: sh
-
-    bokeh serve app_script.py --log-level debug
-
-The available log levels are: {LOGLEVELS}
-
-The log format can be controlled by the ``--log-format`` argument:
-
-.. code-block:: sh
-
-    bokeh serve app_script.py --log-format "%(levelname)s: %(message)s"
-
-The default log format is ``"{DEFAULT_LOG_FORMAT}"``
-
 '''
 
 #-----------------------------------------------------------------------------
@@ -320,6 +413,7 @@ log = logging.getLogger(__name__)
 import argparse
 from fnmatch import fnmatch
 import os
+from glob import glob
 
 # External imports
 from tornado.autoreload import watch
@@ -329,6 +423,7 @@ from bokeh.application import Application
 from bokeh.resources import DEFAULT_SERVER_PORT
 from bokeh.util.logconfig import basicConfig
 from bokeh.util.string import nice_join, format_docstring
+from bokeh.server.auth_provider import AuthModule, NullAuth
 from bokeh.server.tornado import DEFAULT_WEBSOCKET_MAX_MESSAGE_SIZE_BYTES
 from bokeh.settings import settings
 
@@ -361,8 +456,8 @@ base_serve_args = (
     ('--log-level', dict(
         metavar = 'LOG-LEVEL',
         action  = 'store',
-        default = 'info',
-        choices = LOGLEVELS,
+        default = None,
+        choices = LOGLEVELS + (None,),
         help    = "One of: %s" % nice_join(LOGLEVELS),
     )),
 
@@ -379,6 +474,14 @@ base_serve_args = (
         default = None,
         help    = "A filename to write logs to, or None to write to the standard stream (default: None)",
     )),
+
+    ('--use-config', dict(
+        metavar = 'CONFIG',
+        type    = str,
+        help    = "Use a YAML config file for settings",
+        default = None,
+    )),
+
 )
 
 __all__ = (
@@ -401,18 +504,18 @@ class Serve(Subcommand):
 
     args = base_serve_args + (
         ('files', dict(
-            metavar='DIRECTORY-OR-SCRIPT',
-            nargs='*',
-            help="The app directories or scripts to serve (serve empty document if not specified)",
-            default=None,
+            metavar = 'DIRECTORY-OR-SCRIPT',
+            nargs   = '*',
+            help    = "The app directories or scripts to serve (serve empty document if not specified)",
+            default = None,
         )),
 
         ('--args', dict(
-            metavar='COMMAND-LINE-ARGS',
-            nargs=argparse.REMAINDER,
-            help="Command line arguments remaining to passed on to the application handler. "
-                 "NOTE: if this argument precedes DIRECTORY-OR-SCRIPT then some other argument, e.g "
-                 "--show, must be placed before the directory or script. ",
+            metavar = 'COMMAND-LINE-ARGS',
+            nargs   = argparse.REMAINDER,
+            help    = "Command line arguments remaining to passed on to the application handler. "
+                      "NOTE: if this argument precedes DIRECTORY-OR-SCRIPT then some other argument, e.g. "
+                      "--show, must be placed before the directory or script. ",
         )),
 
         ('--dev', dict(
@@ -424,7 +527,7 @@ class Serve(Subcommand):
             help    = "Enable live reloading during app development. "
                       "By default it watches all *.py *.html *.css *.yaml files "
                       "in the app directory tree. Additional files can be passed "
-                      "as arguments."
+                      "as arguments. "
                       "NOTE: if this argument precedes DIRECTORY-OR-SCRIPT then some other argument, e.g "
                       "--show, must be placed before the directory or script. "
                       "NOTE: This setting only works with a single app. "
@@ -435,74 +538,110 @@ class Serve(Subcommand):
         )),
 
         ('--show', dict(
-            action='store_true',
-            help="Open server app(s) in a browser",
+            action = 'store_true',
+            help   = "Open server app(s) in a browser",
         )),
 
         ('--allow-websocket-origin', dict(
-            metavar='HOST[:PORT]',
-            action='append',
-            type=str,
-            help="Public hostnames which may connect to the Bokeh websocket",
+            metavar = 'HOST[:PORT]',
+            action  = 'append',
+            type    = str,
+            help    = "Public hostnames which may connect to the Bokeh websocket",
         )),
 
         ('--prefix', dict(
-            metavar='PREFIX',
-            type=str,
-            help="URL prefix for Bokeh server URLs",
-            default=None,
+            metavar = 'PREFIX',
+            type    = str,
+            help    = "URL prefix for Bokeh server URLs",
+            default = None,
         )),
 
         ('--keep-alive', dict(
-            metavar='MILLISECONDS',
-            type=int,
-            help="How often to send a keep-alive ping to clients, 0 to disable.",
-            default=None,
+            metavar = 'MILLISECONDS',
+            type    = int,
+            help    = "How often to send a keep-alive ping to clients, 0 to disable.",
+            default = None,
         )),
 
         ('--check-unused-sessions', dict(
-            metavar='MILLISECONDS',
-            type=int,
-            help="How often to check for unused sessions",
-            default=None,
+            metavar = 'MILLISECONDS',
+            type    = int,
+            help    = "How often to check for unused sessions",
+            default = None,
         )),
 
         ('--unused-session-lifetime', dict(
-            metavar='MILLISECONDS',
-            type=int,
-            help="How long unused sessions last",
-            default=None,
+            metavar = 'MILLISECONDS',
+            type    = int,
+            help    = "How long unused sessions last",
+            default = None,
         )),
 
         ('--stats-log-frequency', dict(
-            metavar='MILLISECONDS',
-            type=int,
-            help="How often to log stats",
-            default=None,
+            metavar = 'MILLISECONDS',
+            type    = int,
+            help    = "How often to log stats",
+            default = None,
         )),
 
         ('--mem-log-frequency', dict(
-            metavar='MILLISECONDS',
-            type=int,
-            help="How often to log memory usage information",
-            default=None,
+            metavar = 'MILLISECONDS',
+            type    = int,
+            help    = "How often to log memory usage information",
+            default = None,
         )),
 
         ('--use-xheaders', dict(
-            action='store_true',
-            help="Prefer X-headers for IP/protocol information",
+            action = 'store_true',
+            help   = "Prefer X-headers for IP/protocol information",
+        )),
+
+        ('--ssl-certfile', dict(
+            metavar = 'CERTFILE',
+            action  = 'store',
+            default = None,
+            help    = 'Absolute path to a certificate file for SSL termination',
+        )),
+
+        ('--ssl-keyfile', dict(
+            metavar = 'KEYFILE',
+            action  = 'store',
+            default = None,
+            help    = 'Absolute path to a private key file for SSL termination',
         )),
 
         ('--session-ids', dict(
-            metavar='MODE',
+            metavar = 'MODE',
             action  = 'store',
             default = None,
             choices = SESSION_ID_MODES,
             help    = "One of: %s" % nice_join(SESSION_ID_MODES),
         )),
 
+        ('--auth-module', dict(
+            metavar = 'AUTH_MODULE',
+            action  = 'store',
+            default = None,
+            help    = 'Absolute path to a Python module that implements auth hooks',
+        )),
+
+        ('--enable-xsrf-cookies', dict(
+            action  = 'store_true',
+            default = False,
+            help    = 'Whether to enable Tornado support for XSRF cookies. All '
+                      'PUT, POST, or DELETE handlers must be properly instrumented '
+                      'when this setting is enabled.'
+        )),
+
+        ('--cookie-secret', dict(
+            metavar = 'COOKIE_SECRET',
+            action  = 'store',
+            default = None,
+            help    = 'Configure to enable getting/setting secure cookies',
+        )),
+
         ('--index', dict(
-            metavar='INDEX',
+            metavar = 'INDEX',
             action  = 'store',
             default = None,
             help    = 'Path to a template to use for the site index',
@@ -510,30 +649,35 @@ class Serve(Subcommand):
 
         ('--disable-index', dict(
             action = 'store_true',
-            help    = 'Do not use the default index on the root path',
+            help   = 'Do not use the default index on the root path',
         )),
 
         ('--disable-index-redirect', dict(
             action = 'store_true',
-            help    = 'Do not redirect to running app from root path',
+            help   = 'Do not redirect to running app from root path',
         )),
 
         ('--num-procs', dict(
-            metavar='N',
-            action='store',
-            help="Number of worker processes for an app. Using "
-                 "0 will autodetect number of cores (defaults to 1)",
-            default=1,
-            type=int,
+            metavar = 'N',
+            action  = 'store',
+            help    = "Number of worker processes for an app. Using "
+                      "0 will autodetect number of cores (defaults to 1)",
+            default = 1,
+            type    = int,
         )),
 
         ('--websocket-max-message-size', dict(
-            metavar='BYTES',
-            action='store',
-            help="Set the Tornado websocket_max_message_size value (defaults "
-                 "to 20MB) NOTE: This setting has effect ONLY for Tornado>=4.5",
-            default=DEFAULT_WEBSOCKET_MAX_MESSAGE_SIZE_BYTES,
-            type=int,
+            metavar = 'BYTES',
+            action  = 'store',
+            help    = "Set the Tornado websocket_max_message_size value (defaults "
+                      "to 20MB) NOTE: This setting has effect ONLY for Tornado>=4.5",
+            default = DEFAULT_WEBSOCKET_MAX_MESSAGE_SIZE_BYTES,
+            type    = int,
+        )),
+
+        ('--glob', dict(
+            action='store_true',
+            help='Process all filenames as globs',
         )),
     )
 
@@ -541,16 +685,33 @@ class Serve(Subcommand):
         '''
 
         '''
+        basicConfig(format=args.log_format, filename=args.log_file)
+
+        # This is a bit of a fudge. We want the default log level for non-server
+        # cases to be None, i.e. we don't set a log level. But for the server we
+        # do want to set the log level to INFO if nothing else overrides that.
+        log_level = settings.py_log_level(args.log_level)
+        if log_level is None:
+            log_level = logging.INFO
+        logging.getLogger('bokeh').setLevel(log_level)
+
+        if args.use_config is not None:
+            log.info("Using override config file: {}".format(args.use_config))
+            settings.load_config(args.use_config)
 
         # protect this import inside a function so that "bokeh info" can work
         # even if Tornado is not installed
         from bokeh.server.server import Server
 
-        argvs = { f : args.args for f in args.files}
-        applications = build_single_handler_applications(args.files, argvs)
+        files = []
+        for f in args.files:
+            if args.glob:
+                files.extend(glob(f))
+            else:
+                files.append(f)
 
-        log_level = getattr(logging, args.log_level.upper())
-        basicConfig(level=log_level, format=args.log_format, filename=args.log_file)
+        argvs = { f : args.args for f in files}
+        applications = build_single_handler_applications(files, argvs)
 
         if len(applications) == 0:
             # create an empty application by default
@@ -590,6 +751,9 @@ class Serve(Subcommand):
 
         server_kwargs['sign_sessions'] = settings.sign_sessions()
         server_kwargs['secret_key'] = settings.secret_key_bytes()
+        server_kwargs['ssl_certfile'] = settings.ssl_certfile(getattr(args, 'ssl_certfile', None))
+        server_kwargs['ssl_keyfile'] = settings.ssl_keyfile(getattr(args, 'ssl_keyfile', None))
+        server_kwargs['ssl_password'] = settings.ssl_password()
         server_kwargs['generate_session_ids'] = True
         if args.session_ids is None:
             # no --session-ids means use the env vars
@@ -609,6 +773,14 @@ class Serve(Subcommand):
             die("To sign sessions, the BOKEH_SECRET_KEY environment variable must be set; " +
                 "the `bokeh secret` command can be used to generate a new key.")
 
+        auth_module_path = settings.auth_module(getattr(args, 'auth_module', None))
+        if auth_module_path:
+            server_kwargs['auth_provider'] = AuthModule(auth_module_path)
+        else:
+            server_kwargs['auth_provider'] = NullAuth()
+
+        server_kwargs['xsrf_cookies'] = settings.xsrf_cookies(getattr(args, 'enable_xsrf_cookies', False))
+        server_kwargs['cookie_secret'] = settings.cookie_secret(getattr(args, 'cookie_secret', None))
         server_kwargs['use_index'] = not args.disable_index
         server_kwargs['redirect_root'] = not args.disable_index_redirect
         server_kwargs['autoreload'] = args.dev is not None
