@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -17,8 +16,6 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import logging
 log = logging.getLogger(__name__)
 
-from bokeh.util.api import general, dev ; general, dev
-
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
@@ -33,11 +30,15 @@ from bokeh.util.api import general, dev ; general, dev
 # Globals and constants
 #-----------------------------------------------------------------------------
 
+__all__ = (
+    'curdoc',
+    'set_curdoc',
+)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-@general((1,0,0))
 def curdoc():
     ''' Return the document for the current default state.
 
@@ -52,7 +53,6 @@ def curdoc():
 # Dev API
 #-----------------------------------------------------------------------------
 
-@dev((1,0,0))
 def set_curdoc(doc):
     '''Configure the current document (returned by curdoc()).
 
