@@ -36,7 +36,7 @@ from ..model import Model
 
 from .glyphs import Glyph, Circle, MultiLine
 from .graphs import LayoutProvider, GraphHitTestPolicy, NodesOnly
-from .sources import ColumnDataSource, DataSource, RemoteSource, CDSView
+from .sources import ColumnDataSource, DataSource, WebSource, CDSView
 from .tiles import TileSource, WMTSTileSource
 
 #-----------------------------------------------------------------------------
@@ -80,6 +80,18 @@ class DataRenderer(Renderer):
 
     '''
 
+    level = Override(default="glyph")
+
+    x_range_name = String('default', help="""
+    A particular (named) x-range to use for computing screen locations when
+    rendering glyphs on the plot. If unset, use the default x-range.
+    """)
+
+    y_range_name = String('default', help="""
+    A particular (named) y-range to use for computing screen locations when
+    rendering glyphs on the plot. If unset, use the default y-range.
+    """)
+
 class TileRenderer(DataRenderer):
     '''
 
@@ -97,20 +109,6 @@ class TileRenderer(DataRenderer):
     Enable image smoothing for the rendered tiles.
     """)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen
-    locations when rendering glyphs on the plot. If unset, use the
-    default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen
-    locations when rendering glyphs on the plot. If unset, use the
-    default y-range.
-    """)
-
-    level = Override(default="underlay")
-
     render_parents = Bool(default=True, help="""
     Flag enable/disable drawing of parent tiles while waiting for new tiles to arrive. Default value is True.
     """)
@@ -136,7 +134,7 @@ class GlyphRenderer(DataRenderer):
     def _check_bad_column_name(self):
         if not self.glyph: return
         if not self.data_source: return
-        if isinstance(self.data_source, RemoteSource): return
+        if isinstance(self.data_source, WebSource): return
         missing_values = set()
         specs = self.glyph.dataspecs()
         for name, item in self.glyph.properties_with_values(include_defaults=False).items():
@@ -167,18 +165,6 @@ class GlyphRenderer(DataRenderer):
     initialization.
     """)
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen
-    locations when rendering glyphs on the plot. If unset, use the
-    default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen
-    locations when rendering glyphs on the plot. If unset, use the
-    default y-range.
-    """)
-
     glyph = Instance(Glyph, help="""
     The glyph to render, in conjunction with the supplied data source
     and ranges.
@@ -211,8 +197,6 @@ class GlyphRenderer(DataRenderer):
     muted = Bool(False, help="""
     """)
 
-    level = Override(default="glyph")
-
 _DEFAULT_NODE_RENDERER = lambda: GlyphRenderer(
     glyph=Circle(), data_source=ColumnDataSource(data=dict(index=[]))
 )
@@ -238,18 +222,6 @@ class GraphRenderer(DataRenderer):
         if missing:
             return " ,".join(missing) + " [%s]" % self
 
-    x_range_name = String('default', help="""
-    A particular (named) x-range to use for computing screen
-    locations when rendering graphs on the plot. If unset, use the
-    default x-range.
-    """)
-
-    y_range_name = String('default', help="""
-    A particular (named) y-range to use for computing screen
-    locations when rendering graphs on the plot. If unset, use the
-    default y-range.
-    """)
-
     layout_provider = Instance(LayoutProvider, help="""
     An instance of a ``LayoutProvider`` that supplies the layout of the network
     graph in cartesian space.
@@ -275,8 +247,6 @@ class GraphRenderer(DataRenderer):
     of graph components.
     """)
 
-    level = Override(default="glyph")
-
 @abstract
 class GuideRenderer(Renderer):
     ''' A base class for all guide renderer types. ``GuideRenderer`` is
@@ -284,17 +254,6 @@ class GuideRenderer(Renderer):
 
     '''
 
-    plot = Instance(".models.plots.Plot", help="""
-    The plot to which this guide renderer is attached.
-    """)
-
-    def __init__(self, **kwargs):
-        super(GuideRenderer, self).__init__(**kwargs)
-
-        if self.plot is not None:
-            if self not in self.plot.renderers:
-                self.plot.renderers.append(self)
-
     level = Override(default="overlay")
 
 #-----------------------------------------------------------------------------
