@@ -21,7 +21,7 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from datetime import datetime
+from datetime import datetime, date
 import numbers
 
 # External imports
@@ -72,10 +72,6 @@ class AbstractSlider(Widget):
     format = String(help="""
     """)
 
-    orientation = Enum("horizontal", "vertical", help="""
-    Orient the slider either horizontally (default) or vertically.
-    """)
-
     direction = Enum("ltr", "rtl", help="""
     """)
 
@@ -84,6 +80,8 @@ class AbstractSlider(Widget):
 
     callback = Instance(Callback, help="""
     A callback to run in the browser whenever the current Slider value changes.
+
+    DEPRECATED: use .js_on_change or .on_change with "value" or "value_throttled"
     """)
 
     callback_throttle = Float(default=200, help="""
@@ -91,7 +89,9 @@ class AbstractSlider(Widget):
     """)
 
     callback_policy = Enum(SliderCallbackPolicy, default="throttle", help="""
-    When the callback is initiated. This parameter can take on only one of three options:
+    When the value_throttled property is updated.
+
+    This parameter can take on only one of three options:
 
     * "continuous": the callback will be executed immediately for each movement of the slider
     * "throttle": the callback will be executed at most every ``callback_throttle`` milliseconds.
@@ -99,8 +99,7 @@ class AbstractSlider(Widget):
 
     The "mouseup" policy is intended for scenarios in which the callback is expensive in time.
 
-    .. warning::
-        Callback policy currently apply to JS callbacks
+    Both Python and JS callbacks on "value_throttled" will respect this policy setting.
     """)
 
     bar_color = Color(default="#e6e6e6", help="""
@@ -131,6 +130,10 @@ class Slider(AbstractSlider):
     Initial or selected value.
     """)
 
+    value_throttled = Float(help="""
+    Initial or selected value, throttled according to callback_policy.
+    """)
+
     step = Float(default=1, help="""
     The step between consecutive values.
     """)
@@ -144,6 +147,10 @@ class RangeSlider(AbstractSlider):
     Initial or selected range.
     """)
 
+    value_throttled = Tuple(Float, Float, help="""
+    Initial or selected value, throttled according to callback_policy.
+    """)
+
     start = Float(help="""
     The minimum allowable value.
     """)
@@ -165,6 +172,10 @@ class DateSlider(AbstractSlider):
     Initial or selected value.
     """)
 
+    value_throttled = Date(help="""
+    Initial or selected value, throttled according to callback_policy.
+    """)
+
     start = Date(help="""
     The minimum allowable value.
     """)
@@ -199,12 +210,38 @@ class DateRangeSlider(AbstractSlider):
             d2 = datetime.utcfromtimestamp(v2 / 1000)
         else:
             d2 = v2
+        return d1, d2    \
+
+    @property
+    def value_as_date(self):
+        ''' Convenience property to retrieve the value tuple as a tuple of
+        date objects.
+
+        Added in version 1.1
+        '''
+        if self.value is None:
+            return None
+        v1, v2 = self.value
+        if isinstance(v1, numbers.Number):
+            dt = datetime.utcfromtimestamp(v1 / 1000)
+            d1 = date(*dt.timetuple()[:3])
+        else:
+            d1 = v1
+        if isinstance(v2, numbers.Number):
+            dt = datetime.utcfromtimestamp(v2 / 1000)
+            d2 = date(*dt.timetuple()[:3])
+        else:
+            d2 = v2
         return d1, d2
 
     value = Tuple(Date, Date, help="""
     Initial or selected range.
     """)
 
+    value_throttled = Tuple(Date, Date, help="""
+    Initial or selected value, throttled according to callback_policy.
+    """)
+
     start = Date(help="""
     The minimum allowable value.
     """)
