@@ -1,24 +1,51 @@
-from __future__ import absolute_import
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import logging
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+from collections import OrderedDict
+from bokeh.util.future import collections_abc # goes away with py2
+Iterable = collections_abc.Iterable # NOQA
+Sequence = collections_abc.Sequence # NOQA
 
-from collections import Iterable, OrderedDict, Sequence
 import difflib
 import itertools
 import re
 import textwrap
 import warnings
 
+# External imports
 import numpy as np
 import sys
 from six import string_types, reraise
 
+# Bokeh imports
 from ..models import (
-    BoxSelectTool, BoxZoomTool, CategoricalAxis,
+    BoxSelectTool, BoxZoomTool, CategoricalAxis, MercatorAxis,
     TapTool, CrosshairTool, DataRange1d, DatetimeAxis,
     FactorRange, Grid, HelpTool, HoverTool, LassoSelectTool, Legend, LegendItem, LinearAxis,
     LogAxis, PanTool, ZoomInTool, ZoomOutTool, PolySelectTool, ContinuousTicker,
-    SaveTool, Range, Range1d, UndoTool, RedoTool, ResetTool, ResizeTool, Tool,
+    SaveTool, Range, Range1d, UndoTool, RedoTool, ResetTool, Tool,
     WheelPanTool, WheelZoomTool, ColumnarDataSource, ColumnDataSource,
-    LogScale, LinearScale, CategoricalScale, Circle, MultiLine)
+    LogScale, LinearScale, CategoricalScale, Circle, MultiLine,
+    BoxEditTool, PointDrawTool, PolyDrawTool, PolyEditTool,
+)
+from ..models.markers import Marker
 from ..models.renderers import GlyphRenderer
 
 from ..core.properties import ColorSpec, Datetime, value, field
@@ -27,10 +54,83 @@ from ..util.dependencies import import_optional
 from ..util.deprecation import deprecated
 from ..util.string import nice_join
 
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
 pd = import_optional('pandas')
 
+__all__ = (
+    'get_default_color',
+)
 
-def _stack(stackers, spec0, spec1, **kw):
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+def get_default_color(plot=None):
+    colors = [
+        "#1f77b4",
+        "#ff7f0e", "#ffbb78",
+        "#2ca02c", "#98df8a",
+        "#d62728", "#ff9896",
+        "#9467bd", "#c5b0d5",
+        "#8c564b", "#c49c94",
+        "#e377c2", "#f7b6d2",
+        "#7f7f7f",
+        "#bcbd22", "#dbdb8d",
+        "#17becf", "#9edae5"
+    ]
+    if plot:
+        renderers = plot.renderers
+        renderers = [x for x in renderers if x.__view_model__ == "GlyphRenderer"]
+        num_renderers = len(renderers)
+        return colors[num_renderers]
+    else:
+        return colors[0]
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+def _single_stack(stackers, spec, **kw):
+    if spec in kw:
+        raise ValueError("Stack property '%s' cannot appear in keyword args" % spec)
+
+    lengths = { len(x) for x in kw.values() if isinstance(x, (list, tuple)) }
+
+    # lengths will be empty if there are no kwargs supplied at all
+    if len(lengths) > 0:
+        if len(lengths) != 1:
+            raise ValueError("Keyword argument sequences for broadcasting must all be the same lengths. Got lengths: %r" % sorted(list(lengths)))
+        if lengths.pop() != len(stackers):
+            raise ValueError("Keyword argument sequences for broadcasting must be the same length as stackers")
+
+    s = []
+
+    _kw = []
+
+    for i, val in enumerate(stackers):
+        d  = {'name': val}
+        s.append(val)
+
+        d[spec] = stack(*s)
+
+        for k, v in kw.items():
+            if isinstance(v, (list, tuple)):
+                d[k] = v[i]
+            else:
+                d[k] = v
+
+        _kw.append(d)
+
+    return _kw
+
+def _double_stack(stackers, spec0, spec1, **kw):
     for name in (spec0, spec1):
         if name in kw:
             raise ValueError("Stack property '%s' cannot appear in keyword args" % name)
@@ -50,7 +150,7 @@ def _stack(stackers, spec0, spec1, **kw):
     _kw = []
 
     for i, val in enumerate(stackers):
-        d  = {}
+        d  = {'name': val}
         s0 = list(s1)
         s1.append(val)
 
@@ -171,27 +271,6 @@ def _graph(node_source, edge_source, **kwargs):
 
     return renderer_kwargs
 
-def get_default_color(plot=None):
-    colors = [
-        "#1f77b4",
-        "#ff7f0e", "#ffbb78",
-        "#2ca02c", "#98df8a",
-        "#d62728", "#ff9896",
-        "#9467bd", "#c5b0d5",
-        "#8c564b", "#c49c94",
-        "#e377c2", "#f7b6d2",
-        "#7f7f7f",
-        "#bcbd22", "#dbdb8d",
-        "#17becf", "#9edae5"
-    ]
-    if plot:
-        renderers = plot.renderers
-        renderers = [x for x in renderers if x.__view_model__ == "GlyphRenderer"]
-        num_renderers = len(renderers)
-        return colors[num_renderers]
-    else:
-        return colors[0]
-
 
 _RENDERER_ARGS = ['name', 'x_range_name', 'y_range_name',
                   'level', 'view', 'visible', 'muted']
@@ -237,46 +316,57 @@ def _pop_colors_and_alpha(glyphclass, kwargs, prefix="", default_alpha=1.0):
     return result
 
 
-def _get_legend_item_label(kwargs):
-    legend = kwargs.pop('legend', None)
-    source = kwargs.get('source')
-    legend_item_label = None
-    if legend:
-        if isinstance(legend, string_types):
-            # Do the simple thing first
-            legend_item_label = value(legend)
-            # But if there's a source - try and do something smart
-            if source and hasattr(source, 'column_names'):
-                if legend in source.column_names:
-                    legend_item_label = field(legend)
-        else:
-            legend_item_label = legend
-    return legend_item_label
+_LEGEND_ARGS = ['legend', 'legend_label', 'legend_field', 'legend_group']
+
+def _pop_legend_kwarg(kwargs):
+    result = {attr: kwargs.pop(attr) for attr in _LEGEND_ARGS if attr in kwargs}
+    if len(result) > 1:
+        raise ValueError("Only one of %s may be provided, got: %s" % (nice_join(_LEGEND_ARGS), nice_join(result.keys())))
+    return result
 
 
 _GLYPH_SOURCE_MSG = """
-Supplying a user-defined data source AND iterable values to glyph methods is deprecated.
 
-See https://github.com/bokeh/bokeh/issues/2056 for more information.
+Expected %s to reference fields in the supplied data source.
+
+When a 'source' argument is passed to a glyph method, values that are sequences
+(like lists or arrays) must come from references to data columns in the source.
+
+For instance, as an example:
+
+    source = ColumnDataSource(data=dict(x=a_list, y=an_array))
+
+    p.circle(x='x', y='y', source=source, ...) # pass column names and a source
+
+Alternatively, *all* data sequences may be provided as literals as long as a
+source is *not* provided:
+
+    p.circle(x=a_list, y=an_array, ...)  # pass actual sequences and no source
+
 """
 
 
 def _process_sequence_literals(glyphclass, kwargs, source, is_user_source):
+    incompatible_literal_spec_values = []
     dataspecs = glyphclass.dataspecs_with_props()
     for var, val in kwargs.items():
 
         # ignore things that are not iterable
         if not isinstance(val, Iterable):
             continue
+
         # pass dicts (i.e., values or fields) on as-is
         if isinstance(val, dict):
             continue
+
         # let any non-dataspecs do their own validation (e.g., line_dash properties)
         if var not in dataspecs:
             continue
+
         # strings sequences are handled by the dataspec as-is
         if isinstance(val, string_types):
             continue
+
         # similarly colorspecs handle color tuple sequences as-is
         if (isinstance(dataspecs[var].property, ColorSpec) and isinstance(val, tuple)):
             continue
@@ -285,10 +375,12 @@ def _process_sequence_literals(glyphclass, kwargs, source, is_user_source):
             raise RuntimeError("Columns need to be 1D (%s is not)" % var)
 
         if is_user_source:
-            deprecated(_GLYPH_SOURCE_MSG)
+            incompatible_literal_spec_values.append(var)
+        else:
+            source.add(val, name=var)
+            kwargs[var] = var
 
-        source.add(val, name=var)
-        kwargs[var] = var
+    return incompatible_literal_spec_values
 
 
 def _make_glyph(glyphclass, kws, extra):
@@ -299,36 +391,104 @@ def _make_glyph(glyphclass, kws, extra):
     return glyphclass(**kws)
 
 
-def _update_legend(plot, legend_item_label, glyph_renderer):
-    # Get the plot's legend
+def _get_or_create_legend(plot):
     legends = plot.select(type=Legend)
     if not legends:
         legend = Legend()
         plot.add_layout(legend)
-    elif len(legends) == 1:
-        legend = legends[0]
-    else:
-        raise RuntimeError("Plot %s configured with more than one legend renderer" % plot)
+        return legend
+    if len(legends) == 1:
+        return legends[0]
+    raise RuntimeError("Plot %s configured with more than one legend renderer, cannot use legend_* convenience arguments" % plot)
 
-    # If there is an existing legend with a matching label, then put the
-    # renderer on that (if the source matches). Otherwise add a new one.
-    added = False
+
+def _find_legend_item(label, legend):
     for item in legend.items:
-        if item.label == legend_item_label:
-            if item.label.get('value'):
-                item.renderers.append(glyph_renderer)
-                added = True
-                break
-            if item.label.get('field') and \
-                    glyph_renderer.data_source is item.renderers[0].data_source:
-                item.renderers.append(glyph_renderer)
-                added = True
-                break
-    if not added:
-        new_item = LegendItem(label=legend_item_label, renderers=[glyph_renderer])
+        if item.label == label:
+            return item
+    return None
+
+
+def _handle_legend_deprecated(label, legend, glyph_renderer):
+    deprecated("'legend' keyword is deprecated, use explicit 'legend_label', 'legend_field', or 'legend_group' keywords instead")
+
+    if not isinstance(label, (string_types, dict)):
+        raise ValueError("Bad 'legend' parameter value: %s" % label)
+
+    if isinstance(label, dict):
+        if "field" in label and len(label) == 1:
+            label = label['field']
+            _handle_legend_field(label, legend, glyph_renderer)
+        elif "value" in label and len(label) == 1:
+            label = label['value']
+            _handle_legend_label(label, legend, glyph_renderer)
+
+        else:
+            raise ValueError("Bad 'legend' parameter value: %s" % label)
+    else:
+        source = glyph_renderer.data_source
+        if source is not None and hasattr(source, 'column_names') and label in source.column_names:
+            _handle_legend_field(label, legend, glyph_renderer)
+        else:
+            _handle_legend_label(label, legend, glyph_renderer)
+
+
+def _handle_legend_field(label, legend, glyph_renderer):
+    if not isinstance(label, string_types):
+        raise ValueError("legend_field value must be a string")
+    label = field(label)
+    item = _find_legend_item(label, legend)
+    if item:
+        item.renderers.append(glyph_renderer)
+    else:
+        new_item = LegendItem(label=label, renderers=[glyph_renderer])
         legend.items.append(new_item)
 
 
+def _handle_legend_group(label, legend, glyph_renderer):
+    if not isinstance(label, string_types):
+        raise ValueError("legend_group value must be a string")
+
+    source = glyph_renderer.data_source
+    if source is None:
+        raise ValueError("Cannot use 'legend_group' on a glyph without a data source already configured")
+    if not (hasattr(source, 'column_names') and label in source.column_names):
+        raise ValueError("Column to be grouped does not exist in glyph data source")
+
+    column = source.data[label]
+    vals, inds = np.unique(column, return_index=1)
+    for val, ind in zip(vals, inds):
+        label = value(str(val))
+        new_item = LegendItem(label=label, renderers=[glyph_renderer], index=ind)
+        legend.items.append(new_item)
+
+
+def _handle_legend_label(label, legend, glyph_renderer):
+    if not isinstance(label, string_types):
+        raise ValueError("legend_label value must be a string")
+    label = value(label)
+    item = _find_legend_item(label, legend)
+    if item:
+        item.renderers.append(glyph_renderer)
+    else:
+        new_item = LegendItem(label=label, renderers=[glyph_renderer])
+        legend.items.append(new_item)
+
+
+_LEGEND_KWARG_HANDLERS = {
+    'legend'       : _handle_legend_deprecated,
+    'legend_label' : _handle_legend_label,
+    'legend_field' : _handle_legend_field,
+    'legend_group' : _handle_legend_group,
+}
+
+def _update_legend(plot, legend_kwarg, glyph_renderer):
+    legend = _get_or_create_legend(plot)
+    kwarg, value = list(legend_kwarg.items())[0]
+
+    _LEGEND_KWARG_HANDLERS[kwarg](value, legend, glyph_renderer)
+
+
 def _get_range(range_input):
     if range_input is None:
         return DataRange1d()
@@ -336,7 +496,9 @@ def _get_range(range_input):
         return FactorRange(factors=sorted(list(range_input.groups.keys())))
     if isinstance(range_input, Range):
         return range_input
-    if isinstance(range_input, Sequence):
+    if pd and isinstance(range_input, pd.Series):
+        range_input = range_input.values
+    if isinstance(range_input, (Sequence, np.ndarray)):
         if all(isinstance(x, string_types) for x in range_input):
             return FactorRange(factors=list(range_input))
         if len(range_input) == 2:
@@ -348,7 +510,7 @@ def _get_range(range_input):
 
 
 def _get_scale(range_input, axis_type):
-    if isinstance(range_input, (DataRange1d, Range1d)) and axis_type in ["linear", "datetime", "auto", None]:
+    if isinstance(range_input, (DataRange1d, Range1d)) and axis_type in ["linear", "datetime", "mercator", "auto", None]:
         return LinearScale()
     elif isinstance(range_input, (DataRange1d, Range1d)) and axis_type == "log":
         return LogScale()
@@ -358,26 +520,28 @@ def _get_scale(range_input, axis_type):
         raise ValueError("Unable to determine proper scale for: '%s'" % str(range_input))
 
 
-def _get_axis_class(axis_type, range_input):
+def _get_axis_class(axis_type, range_input, dim):
     if axis_type is None:
-        return None
+        return None, {}
     elif axis_type == "linear":
-        return LinearAxis
+        return LinearAxis, {}
     elif axis_type == "log":
-        return LogAxis
+        return LogAxis, {}
     elif axis_type == "datetime":
-        return DatetimeAxis
+        return DatetimeAxis, {}
+    elif axis_type == "mercator":
+        return MercatorAxis, {'dimension': 'lon' if dim == 0 else 'lat'}
     elif axis_type == "auto":
         if isinstance(range_input, FactorRange):
-            return CategoricalAxis
+            return CategoricalAxis, {}
         elif isinstance(range_input, Range1d):
             try:
                 # Easier way to validate type of Range1d parameters
                 Datetime.validate(Datetime(), range_input.start)
-                return DatetimeAxis
+                return DatetimeAxis, {}
             except ValueError:
                 pass
-        return LinearAxis
+        return LinearAxis, {}
     else:
         raise ValueError("Unrecognized axis_type: '%r'" % axis_type)
 
@@ -398,6 +562,8 @@ _known_tools = {
     "pan": lambda: PanTool(dimensions='both'),
     "xpan": lambda: PanTool(dimensions='width'),
     "ypan": lambda: PanTool(dimensions='height'),
+    "xwheel_pan": lambda: WheelPanTool(dimension="width"),
+    "ywheel_pan": lambda: WheelPanTool(dimension="height"),
     "wheel_zoom": lambda: WheelZoomTool(dimensions='both'),
     "xwheel_zoom": lambda: WheelZoomTool(dimensions='width'),
     "ywheel_zoom": lambda: WheelZoomTool(dimensions='height'),
@@ -407,9 +573,6 @@ _known_tools = {
     "zoom_out": lambda: ZoomOutTool(dimensions='both'),
     "xzoom_out": lambda: ZoomOutTool(dimensions='width'),
     "yzoom_out": lambda: ZoomOutTool(dimensions='height'),
-    "xwheel_pan": lambda: WheelPanTool(dimension="width"),
-    "ywheel_pan": lambda: WheelPanTool(dimension="height"),
-    "resize": lambda: ResizeTool(),
     "click": lambda: TapTool(behavior="inspect"),
     "tap": lambda: TapTool(),
     "crosshair": lambda: CrosshairTool(),
@@ -424,7 +587,7 @@ _known_tools = {
     "hover": lambda: HoverTool(tooltips=[
         ("index", "$index"),
         ("data (x, y)", "($x, $y)"),
-        ("canvas (x, y)", "($sx, $sy)"),
+        ("screen (x, y)", "($sx, $sy)"),
     ]),
     "save": lambda: SaveTool(),
     "previewsave": "save",
@@ -432,6 +595,10 @@ _known_tools = {
     "redo": lambda: RedoTool(),
     "reset": lambda: ResetTool(),
     "help": lambda: HelpTool(),
+    "box_edit": lambda: BoxEditTool(),
+    "point_draw": lambda: PointDrawTool(),
+    "poly_draw": lambda: PolyDrawTool(),
+    "poly_edit": lambda: PolyEditTool()
 }
 
 
@@ -456,19 +623,10 @@ def _tool_from_string(name):
 
 
 def _process_axis_and_grid(plot, axis_type, axis_location, minor_ticks, axis_label, rng, dim):
-    axiscls = _get_axis_class(axis_type, rng)
+    axiscls, axiskw = _get_axis_class(axis_type, rng, dim)
+
     if axiscls:
-
-        if axiscls is LogAxis:
-            if dim == 0:
-                plot.x_scale = LogScale()
-            elif dim == 1:
-                plot.y_scale = LogScale()
-            else:
-                raise ValueError("received invalid dimension value: %r" % dim)
-
-        # this is so we can get a ticker off the axis, even if we discard it
-        axis = axiscls(plot=plot if axis_location else None)
+        axis = axiscls(**axiskw)
 
         if isinstance(axis.ticker, ContinuousTicker):
             axis.ticker.num_minor_ticks = _get_num_minor_ticks(axiscls, minor_ticks)
@@ -477,13 +635,14 @@ def _process_axis_and_grid(plot, axis_type, axis_location, minor_ticks, axis_lab
         if axis_label:
             axis.axis_label = axis_label
 
-        grid = Grid(plot=plot, dimension=dim, ticker=axis.ticker); grid
+        grid = Grid(dimension=dim, ticker=axis.ticker)
+        plot.add_layout(grid, "center")
 
         if axis_location is not None:
             getattr(plot, axis_location).append(axis)
 
 
-def _process_tools_arg(plot, tools):
+def _process_tools_arg(plot, tools, tooltips=None):
     """ Adds tools to the plot object
 
     Args:
@@ -491,6 +650,8 @@ def _process_tools_arg(plot, tools):
         tools (seq[Tool or str]|str): list of tool types or string listing the
             tool names. Those are converted using the _tool_from_string
             function. I.e.: `wheel_zoom,box_zoom,reset`.
+        tooltips (string or seq[tuple[str, str]], optional):
+            tooltips to use to configure a HoverTool
 
     Returns:
         list of Tools objects added to plot, map of supplied string names to tools
@@ -520,13 +681,21 @@ def _process_tools_arg(plot, tools):
         tool_map[tool] = tool_obj
 
     for typename, group in itertools.groupby(
-            sorted([tool.__class__.__name__ for tool in tool_objs])):
+            sorted(tool.__class__.__name__ for tool in tool_objs)):
         if len(list(group)) > 1:
             repeated_tools.append(typename)
 
     if repeated_tools:
         warnings.warn("%s are being repeated" % ",".join(repeated_tools))
 
+    if tooltips is not None:
+        for tool_obj in tool_objs:
+            if isinstance(tool_obj, HoverTool):
+                tool_obj.tooltips = tooltips
+                break
+        else:
+            tool_objs.append(HoverTool(tooltips=tooltips))
+
     return tool_objs, tool_map
 
 
@@ -554,7 +723,7 @@ def _process_active_tools(toolbar, tool_map, active_drag, active_inspect, active
     else:
         raise ValueError("Got unknown %r for 'active_drag', which was not a string supplied in 'tools' argument" % active_drag)
 
-    if active_inspect in ['auto', None] or isinstance(active_inspect, Tool) or all([isinstance(t, Tool) for t in active_inspect]):
+    if active_inspect in ['auto', None] or isinstance(active_inspect, Tool) or all(isinstance(t, Tool) for t in active_inspect):
         toolbar.active_inspect = active_inspect
     elif active_inspect in tool_map:
         toolbar.active_inspect = tool_map[active_inspect]
@@ -627,7 +796,7 @@ def _get_sigfunc(func_name, func, argspecs):
 _arg_template = """    %s (%s) : %s
         (default: %r)
 """
-_doc_template = """ Configure and add %s glyphs to this Figure.
+_doc_template = """ Configure and add :class:`~bokeh.models.%s.%s` glyphs to this Figure.
 
 Args:
 %s
@@ -653,7 +822,7 @@ Returns:
 """
 
 def _add_sigfunc_info(func, argspecs, glyphclass, extra_docs):
-    func.__name__ = glyphclass.__name__.lower()
+    func.__name__ = glyphclass.__name__
 
     omissions = {'js_event_callbacks', 'js_property_callbacks', 'subscribed_events'}
 
@@ -679,7 +848,8 @@ def _add_sigfunc_info(func, argspecs, glyphclass, extra_docs):
     for arg, spec in argspecs.items():
         arglines.append(_arg_template % (arg, spec['type'], spec['desc'], spec['default']))
 
-    func.__doc__ = _doc_template % (func.__name__, "\n".join(arglines), "\n".join(kwlines))
+    mod = "markers" if issubclass(glyphclass, Marker) else "glyphs"
+    func.__doc__ = _doc_template % (mod, func.__name__, "\n".join(arglines), "\n".join(kwlines))
     if extra_docs:
         func.__doc__ += extra_docs
 
@@ -687,31 +857,42 @@ def _glyph_function(glyphclass, extra_docs=None):
 
     def func(self, **kwargs):
 
-        # Process legend kwargs and remove legend before we get going
-        legend_item_label = _get_legend_item_label(kwargs)
+        # Convert data source, if necessary
+        is_user_source = kwargs.get('source', None) is not None
+        if is_user_source:
+            source = kwargs['source']
+            if not isinstance(source, ColumnarDataSource):
+                try:
+                    # try converting the soruce to ColumnDataSource
+                    source = ColumnDataSource(source)
+                except ValueError as err:
+                    msg = "Failed to auto-convert {curr_type} to ColumnDataSource.\n Original error: {err}".format(
+                        curr_type=str(type(source)),
+                        err=err.message
+                    )
+                    reraise(ValueError, ValueError(msg), sys.exc_info()[2])
+
+                # update reddered_kws so that others can use the new source
+                kwargs['source'] = source
+
+        # Save off legend kwargs before we get going
+        legend_kwarg = _pop_legend_kwarg(kwargs)
 
         # Need to check if user source is present before _pop_renderer_args
-        is_user_source = kwargs.get('source', None) is not None
         renderer_kws = _pop_renderer_args(kwargs)
         source = renderer_kws['data_source']
-        if not isinstance(source, ColumnarDataSource):
-            try:
-                # try converting the soruce to ColumnDataSource
-                source = ColumnDataSource(source)
-            except ValueError as err:
-                msg = "Failed to auto-convert {curr_type} to ColumnDataSource.\n Original error: {err}".format(
-                    curr_type=str(type(source)),
-                    err=err.message
-                )
-                reraise(ValueError, ValueError(msg), sys.exc_info()[2])
 
-            # update reddered_kws so that others can use the new source
-            renderer_kws['data_source'] = source
+        # Assign global_alpha from alpha if glyph type is an image
+        if 'alpha' in kwargs and glyphclass.__name__ in ('Image', 'ImageRGBA', 'ImageURL'):
+            kwargs['global_alpha'] = kwargs['alpha']
 
         # handle the main glyph, need to process literals
         glyph_ca = _pop_colors_and_alpha(glyphclass, kwargs)
-        _process_sequence_literals(glyphclass, kwargs, source, is_user_source)
-        _process_sequence_literals(glyphclass, glyph_ca, source, is_user_source)
+        incompatible_literal_spec_values = []
+        incompatible_literal_spec_values += _process_sequence_literals(glyphclass, kwargs, source, is_user_source)
+        incompatible_literal_spec_values += _process_sequence_literals(glyphclass, glyph_ca, source, is_user_source)
+        if incompatible_literal_spec_values:
+            raise RuntimeError(_GLYPH_SOURCE_MSG % nice_join(incompatible_literal_spec_values, conjuction="and"))
 
         # handle the nonselection glyph, we always set one
         nsglyph_ca = _pop_colors_and_alpha(glyphclass, kwargs, prefix='nonselection_', default_alpha=0.1)
@@ -747,11 +928,8 @@ def _glyph_function(glyphclass, extra_docs=None):
                                        muted_glyph=mglyph,
                                        **renderer_kws)
 
-        if legend_item_label:
-            _update_legend(self, legend_item_label, glyph_renderer)
-
-        for tool in self.select(type=BoxSelectTool):
-            tool.renderers.append(glyph_renderer)
+        if legend_kwarg:
+            _update_legend(self, legend_kwarg, glyph_renderer)
 
         self.renderers.append(glyph_renderer)
 
@@ -766,3 +944,7 @@ def _glyph_function(glyphclass, extra_docs=None):
     _add_sigfunc_info(sigfunc, argspecs, glyphclass, extra_docs)
 
     return sigfunc
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
