@@ -29,7 +29,7 @@ from tornado import gen
 # Bokeh imports
 from bokeh.core.templates import AUTOLOAD_JS
 from bokeh.util.string import encode_utf8
-from bokeh.util.compiler import bundle_all_models
+from bokeh.embed.bundle import bundle_for_objs_and_resources, Script
 from bokeh.embed.elements import script_for_render_items
 from bokeh.embed.util import RenderItem
 
@@ -68,31 +68,18 @@ class AutoloadJsHandler(SessionHandler):
         absolute_url = self.get_argument("bokeh-absolute-url", default=None)
 
         if absolute_url:
-            server_url = '{uri.scheme}://{uri.netloc}/'.format(uri=urlparse(absolute_url))
+            server_url = '{uri.scheme}://{uri.netloc}'.format(uri=urlparse(absolute_url))
         else:
             server_url = None
-        resources = self.application.resources(server_url)
-
-        bundle = bundle_all_models()
-
-        render_items = [RenderItem(sessionid=session.id, elementid=element_id, use_for_title=False)]
-        script = script_for_render_items(None, render_items, app_path=app_path, absolute_url=absolute_url)
 
         resources_param = self.get_argument("resources", "default")
-        if resources_param == "none":
-            js_urls = []
-            css_urls = []
-        else:
-            js_urls = resources.js_files
-            css_urls = resources.css_files
+        resources = self.application.resources(server_url) if resources_param != "none" else None
+        bundle = bundle_for_objs_and_resources(None, resources)
 
-        js = AUTOLOAD_JS.render(
-            js_urls = js_urls,
-            css_urls = css_urls,
-            js_raw = resources.js_raw + [bundle, script],
-            css_raw = resources.css_raw_str,
-            elementid = element_id,
-        )
+        render_items = [RenderItem(sessionid=session.id, elementid=element_id, use_for_title=False)]
+        bundle.add(Script(script_for_render_items(None, render_items, app_path=app_path, absolute_url=absolute_url)))
+
+        js = AUTOLOAD_JS.render(bundle=bundle, elementid=element_id)
 
         self.set_header("Content-Type", 'application/javascript')
         self.write(encode_utf8(js))
