@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -20,12 +19,14 @@ import pytest ; pytest
 # Standard library imports
 from os.path import dirname, join
 from shutil import copy
+import warnings
 
 # External imports
 from six import string_types
 
 # Bokeh imports
-from bokeh.util.testing import verify_all
+from bokeh._testing.util.api import verify_all
+from bokeh.util.warnings import BokehDeprecationWarning, BokehUserWarning
 
 # Module under test
 import bokeh as b
@@ -41,7 +42,7 @@ ALL =  (
 )
 
 _LICENSE = """\
-Copyright (c) 2012, Anaconda, Inc.
+Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification,
@@ -95,3 +96,29 @@ def test_license(capsys):
     b.license()
     out, err = capsys.readouterr()
     assert out == _LICENSE
+
+class TestWarnings(object):
+    @pytest.mark.parametrize('cat', (BokehDeprecationWarning, BokehUserWarning))
+    def test_bokeh_custom(self, cat):
+        r = warnings.formatwarning("message", cat, "line", "lineno")
+        assert r == "%s: %s\n" %(cat.__name__, "message")
+
+    def test_general_default(self):
+        r = warnings.formatwarning("message", RuntimeWarning, "line", "lineno")
+        assert r == "line:lineno: RuntimeWarning: message\n"
+
+    def test_filters(self):
+        assert ('always', None, BokehUserWarning, None, 0) in warnings.filters
+        assert ('always', None, BokehDeprecationWarning, None, 0) in warnings.filters
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
